@@ -8,14 +8,16 @@
 
 from arpeggio import Parser, Sequence, NoMatch, EOF, Terminal
 from .exceptions import TextXSyntaxError, TextXSemanticError
-from .const import MULT_ONEORMORE, MULT_ZEROORMORE
+from .const import MULT_ONEORMORE, MULT_ZEROORMORE, RULE_NORMAL,\
+    RULE_ABSTRACT
+
 
 def convert(value, _type):
     """
     Convert instances of textx types to python types.
     """
     return {
-            'BOOL'  : lambda x: x=='1' or x.lower()=='true',
+            'BOOL'  : lambda x: x == '1' or x.lower() == 'true',
             'INT'   : lambda x: int(x),
             'FLOAT' : lambda x: float(x),
             'STRING': lambda x: x.strip('"\''),
@@ -29,7 +31,7 @@ class ObjCrossRef(object):
     Attributes:
         obj_name(str): A name of the target object.
         cls(TextXClass): The target object class.
-        position(int): A position in the input stirng of this cross-ref.
+        position(int): A position in the input string of this cross-ref.
     """
     def __init__(self, obj_name, cls, position):
         self.obj_name = obj_name
@@ -53,13 +55,15 @@ def get_model_parser(top_rule, comments_model, debug=False):
 
             # By default first rule is starting rule
             # and must be followed by the EOF
-            self.parser_model = Sequence(nodes=[top_rule, EOF()],\
-                    rule_name='ModelFile', root=True)
+            self.parser_model = Sequence(
+                nodes=[top_rule, EOF()], rule_name='ModelFile', root=True)
             self.comments_model = comments_model
 
             # Stack for metaclass instances
             self._inst_stack = []
+
             # Dict for cross-ref resolving
+            # { id(class): { obj.name: obj}}
             self._instances = {}
 
             self.debug = debug
@@ -68,17 +72,27 @@ def get_model_parser(top_rule, comments_model, debug=False):
             try:
                 return self.parser_model.parse(self)
             except NoMatch as e:
-                raise TextXSyntaxError(str(e))
+                line, col = e.parser.pos_to_linecol(e.position)
+                raise TextXSyntaxError(str(e), line, col)
 
         def get_model_from_file(self, file_name):
             """
             Creates model from the parse tree from the previous parse call.
-            If file_name is given file will be parsed before model construction.
+            If file_name is given file will be parsed before model
+            construction.
             """
             with open(file_name, 'r') as f:
                 model_str = f.read()
 
-            return self.get_model_from_str(model_str)
+            model = self.get_model_from_str(model_str)
+
+            # Register filename of the model for later use.
+            try:
+                model._filename = file_name
+            except AttributeError:
+                # model is some primitive python type (e.g. str)
+                pass
+            return model
 
         def get_model_from_str(self, model_str):
             """
@@ -90,6 +104,11 @@ def get_model_parser(top_rule, comments_model, debug=False):
             # Transform parse tree to model. Skip root node which
             # represents the whole file ending in EOF.
             model = parse_tree_to_objgraph(self, self.parse_tree[0])
+            try:
+                model._filename = None
+            except AttributeError:
+                # model is some primitive python type (e.g. str)
+                pass
             return model
 
     return TextXModelParser()
@@ -107,7 +126,8 @@ def parse_tree_to_objgraph(parser, parse_tree):
         if isinstance(node, Terminal):
             return convert(node.value, node.rule_name)
 
-        assert node.rule.root, "Not a root node: {}".format(node.rule.rule_name)
+        assert node.rule.root,\
+            "Not a root node: {}".format(node.rule.rule_name)
         # If this node is created by some root rule
         # create metaclass instance.
         inst = None
@@ -125,11 +145,13 @@ def parse_tree_to_objgraph(parser, parse_tree):
                 print("CREATING INSTANCE {}".format(node.rule_name))
 
             inst = mclass()
+            inst._position = node.position
             parser._inst_stack.append(inst)
 
             for n in node:
                 if parser.debug:
-                    print("Recursing into {} = '{}'".format(type(n).__name__, str(n)))
+                    print("Recursing into {} = '{}'"
+                          .format(type(n).__name__, str(n)))
                 process_node(n)
 
             parser._inst_stack.pop()
@@ -143,8 +165,6 @@ def parse_tree_to_objgraph(parser, parse_tree):
                 parser._instances[id(inst.__class__)][inst.name] = inst
 
             if parser.debug:
-                old_str = "{}(name={})".format(type(inst).__name__, inst.name)  \
-                        if hasattr(inst, 'name') else type(inst).__name__
                 print("LEAVING INSTANCE {}".format(node.rule_name))
 
         else:
@@ -164,16 +184,18 @@ def parse_tree_to_objgraph(parser, parse_tree):
             elif op == 'plain':
                 attr_value = getattr(i, attr_name)
                 if attr_value and type(attr_value) is not list:
-                    raise TextXSemanticError("Multiple assignments to attribute {} at {}"\
-                            .format(attr_name, parser.pos_to_linecol(node.position)))
+                    raise TextXSemanticError(
+                        "Multiple assignments to attribute {} at {}"
+                        .format(attr_name,
+                                parser.pos_to_linecol(node.position)))
 
                 # Recurse and convert value to proper type
                 value = process_node(node[0])
 
                 if metaattr.ref and not metaattr.cont:
                     # If this is non-containing reference create ObjCrossRef
-                    value = ObjCrossRef(obj_name=value, cls=metaattr.cls, \
-                            position=node[0].position)
+                    value = ObjCrossRef(obj_name=value, cls=metaattr.cls,
+                                        position=node[0].position)
 
                 if type(attr_value) is list:
                     attr_value.append(value)
@@ -189,11 +211,14 @@ def parse_tree_to_objgraph(parser, parse_tree):
                         value = process_node(n)
 
                         if metaattr.ref and not metaattr.cont:
-                            # If this is non-containing reference create ObjCrossRef
-                            value = ObjCrossRef(obj_name=value, cls=metaattr.cls,\
-                                    position=node[0].position)
+                            # If this is non-containing reference
+                            # create ObjCrossRef
+                            value = ObjCrossRef(obj_name=value,
+                                                cls=metaattr.cls,
+                                                position=node[0].position)
 
-                        if not hasattr(i, attr_name) or getattr(i, attr_name) is None:
+                        if not hasattr(i, attr_name) or \
+                                getattr(i, attr_name) is None:
                             setattr(i, attr_name, [])
                         getattr(i, attr_name).append(value)
             else:
@@ -206,6 +231,8 @@ def parse_tree_to_objgraph(parser, parse_tree):
         """
         Resolves obj cross refs.
         """
+        # TODO: Scoping and name-space rules.
+
         resolved_set = set()
         metamodel = parser.metamodel
 
@@ -214,14 +241,52 @@ def parse_tree_to_objgraph(parser, parse_tree):
                 return
             assert type(obj_ref) is ObjCrossRef, type(obj_ref)
             if parser.debug:
-                print("Resolving obj crossref: {}:{}"\
-                        .format(obj_ref.cls, obj_ref.obj_name))
-            if id(obj_ref.cls) in parser._instances:
-                objs = parser._instances[id(obj_ref.cls)]
-                if obj_ref.obj_name in objs:
-                    return objs[obj_ref.obj_name]
-            raise TextXSemanticError('Unknown object "{}" of class "{}" at {}'\
-                    .format(obj_ref.obj_name, obj_ref.cls.__name__, parser.pos_to_linecol(obj_ref.position)))
+                print("Resolving obj crossref: {}:{}"
+                      .format(obj_ref.cls, obj_ref.obj_name))
+
+            def _resolve_ref_abstract(obj_cls):
+                """
+                Depth-first resolving of abstract rules.
+                """
+                for inherited in obj_cls._inh_by:
+                    if inherited._type == RULE_ABSTRACT:
+                        return _resolve_ref_abstract(inherited)
+                    elif inherited._type == RULE_NORMAL:
+                        if id(inherited) in parser._instances:
+                            objs = parser._instances[id(inherited)]
+                            if obj_ref.obj_name in objs:
+                                return objs[obj_ref.obj_name]
+
+            if obj_ref.cls._type == RULE_NORMAL:
+                if id(obj_ref.cls) in parser._instances:
+                    objs = parser._instances[id(obj_ref.cls)]
+                    if obj_ref.obj_name in objs:
+                        return objs[obj_ref.obj_name]
+            elif obj_ref.cls._type == RULE_ABSTRACT:
+                # For abstract rule ref do a depth first search on
+                # the inheritance tree to find normal rules
+                # and return a first instance of that meta-class instance with
+                # the referred name.
+                obj = _resolve_ref_abstract(obj_ref.cls)
+                if obj:
+                    return obj
+            else:
+                pass
+                # TODO: Match rules cannot be referred. This is
+                #       an error in language description.
+
+
+            # As a fall-back search builtins if given
+            if metamodel.builtins:
+                if obj_ref.obj_name in metamodel.builtins:
+                    # TODO: Classes must match
+                    return metamodel.builtins[obj_ref.obj_name]
+
+            line, col = parser.pos_to_linecol(obj_ref.position)
+            raise TextXSemanticError(
+                'Unknown object "{}" of class "{}" at {}'
+                .format(obj_ref.obj_name, obj_ref.cls.__name__, (line, col)),
+                line=line, col=col)
 
         def _resolve(o):
             if parser.debug:
@@ -230,28 +295,32 @@ def parse_tree_to_objgraph(parser, parse_tree):
                 return
             resolved_set.add(o)
 
-            for attr in o.__class__._attrs.values():
-                if parser.debug:
-                    print("RESOLVING ATTR: {}".format(attr.name))
-                    print("mult={}, ref={}, con={}".format(attr.mult, attr.ref, attr.cont))
-                attr_value = getattr(o, attr.name)
-                if attr.mult in [MULT_ONEORMORE, MULT_ZEROORMORE]:
-                    for idx, list_attr_value in enumerate(attr_value):
+            # If this object has attributes (created using a normal rule)
+            if hasattr(o.__class__, "_attrs"):
+                for attr in o.__class__._attrs.values():
+                    if parser.debug:
+                        print("RESOLVING ATTR: {}".format(attr.name))
+                        print("mult={}, ref={}, con={}".format(
+                            attr.mult,
+                            attr.ref, attr.cont))
+                    attr_value = getattr(o, attr.name)
+                    if attr.mult in [MULT_ONEORMORE, MULT_ZEROORMORE]:
+                        for idx, list_attr_value in enumerate(attr_value):
+                            if attr.ref:
+                                if attr.cont:
+                                    _resolve(list_attr_value)
+                                else:
+                                    attr_value[idx] = \
+                                        _resolve_ref(list_attr_value)
+                    else:
                         if attr.ref:
                             if attr.cont:
-                                _resolve(list_attr_value)
+                                _resolve(attr_value)
                             else:
-                                attr_value[idx] = _resolve_ref(list_attr_value)
-                else:
-                    if attr.ref:
-                        if attr.cont:
-                            _resolve(attr_value)
-                        else:
-                            setattr(o, attr.name, _resolve_ref(attr_value))
+                                setattr(o, attr.name, _resolve_ref(attr_value))
 
         _resolve(model)
 
-
     model = process_node(parse_tree)
     resolve_refs(model)
     assert not parser._inst_stack
